<?php

/**
 * @file
 * Contains \Drupal\restui\Form\RestUIForm.
 */

namespace Drupal\restui\Form;

use Drupal\system\SystemConfigFormBase;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\Context\ContextInterface;
use Drupal\Core\Extension\ModuleHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Authentication\AuthenticationManager;

/**
 * Manage REST resources.
 */
class RestUIForm extends SystemConfigFormBase {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * The authentication manager.
   *
   * @var \Drupal\Core\Authentication\AuthenticationManager
   */
  protected $authenticationManager;

  /**
   * The available serialization formats.
   *
   * @var array
   */
  protected $formats = array();

  /**
   * Constructs a \Drupal\user\RestForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\Context\ContextInterface $context
   *   The configuration context.
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   *   The module handler.
   * @param \Drupal\Core\Authentication\AuthenticationManager $authenticationManager
   *   The Authentication Manager.
   */
  public function __construct(ConfigFactory $config_factory, ContextInterface $context, ModuleHandler $module_handler, AuthenticationManager $authenticationManager, array $serializer_formats) {
    parent::__construct($config_factory, $context);
    $this->moduleHandler = $module_handler;
    $this->authenticationManager = $authenticationManager;
    $this->formats = $serializer_formats;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.context.free'),
      $container->get('module_handler'),
      $container->get('authentication'),
      $container->getParameter('serializer.formats')
    );
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::getFormID().
   */
  public function getFormID() {
    return 'restui';
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::buildForm().
   */
  public function buildForm(array $form, array &$form_state) {
    $resources = drupal_container()
      ->get('plugin.manager.rest')
      ->getDefinitions();

    $plugin_manager = drupal_container()->get('plugin.manager.rest');
    $authentication_providers = array_keys($this->authenticationManager->getSortedProviders());

    $available_resources = array();
    foreach ($resources as $plugin_name => $definition) {
      $plugin = $plugin_manager->getInstance(array('id' => $definition['id']));
      $methods = $plugin->availableMethods();
      // Non-entity resources.
      if (strpos($plugin_name, 'entity:') === FALSE) {
        $available_resources[$plugin_name] = array(
          'name' => $definition['label'],
          'path' => '<code>/' . $definition['id'] . '/{id}</code>',
          'methods' => $methods,
        );
      }
      // Entity resources.
      else {
        $available_resources[$plugin_name] = array(
          'name' => $definition['label'],
          'path' => '<code>/entity/' . $definition['entity_type'] . '/{id}</code>',
          'methods' => $methods,
        );
      }
    }
    asort($available_resources);
    $config = config('rest.settings')->get('resources') ? : array();
    // Strip out the nested method configuration, we are only interested in the
    // plugin IDs of the resources.
    $enabled_resources = drupal_map_assoc(array_keys($config));

    $form['#tree'] = TRUE;
    $form['resources_title'] = array(
      '#markup' => '<h2>' . t('Resource types that should be exposed as web services') . '</h2>',
    );

    $form['resources'] = array('#type' => 'container');

    foreach ($available_resources as $resource_id => $resource) {
      $resource_group = array();
      $resource_group['active'] = array(
        '#type' => 'checkbox',
        '#title' => $resource['name'] . ' (' . $resource['path'] . ')',
        '#default_value' => in_array($resource_id, $enabled_resources),
      );

      // Settings for this resource.
      $resource_group["settings"] = array(
        '#title' => 'Methods',
        '#type' => 'fieldset',
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
        '#states' => array(
          'visible' => array(
            ':input[name="resources[' . $resource_id . '][active]"]' => array('checked' => TRUE),
          ),
        ),
      );

      // HTTP methods.
      foreach ($resource['methods'] as $method) {
        $settings_group = array();
        $settings_group['method'] = array(
          '#title' => $method,
          '#type' => 'checkbox',
          '#default_value' => (isset($config[$resource_id][$method]) || empty($config[$resource_id])),
        );
        $settings_group['method_name'] = array(
          '#type' => 'value',
          '#value' => $method
        );
        // Available formats.
        $enabled_formats = array();
        if (isset($config[$resource_id][$method]['supported_formats'])) {
          $enabled_formats = $config[$resource_id][$method]['supported_formats'];
        }
        $settings_group['supported_formats'] = array(
          '#title' => 'Supported formats',
          '#type' => 'checkboxes',
          '#options' => array_combine($this->formats, $this->formats),
          '#multiple' => TRUE,
          '#default_value' => $enabled_formats,
        );

        // Authentication methods.
        $enabled_auth = array();
        if (isset($config[$resource_id][$method]['supported_auth'])) {
          $enabled_auth = $config[$resource_id][$method]['supported_auth'];
        }
        $settings_group['supported_auth'] = array(
          '#title' => 'Authentication methods',
          '#type' => 'checkboxes',
          '#options' => array_combine($authentication_providers, $authentication_providers),
          '#multiple' => TRUE,
          '#default_value' => $enabled_auth,
        );
        $resource_group["settings"][] = $settings_group;
      }
      $form['resources'][$resource_id] = $resource_group;
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::submitForm().
   */
  public function submitForm(array &$form, array &$form_state) {
    $values = $form_state['values']['resources'];
    $enabled_resources = array();
    $enabled_resources_settings = array();

    foreach ($values as $id => $resource) {
      if ($resource['active']) {
        $enabled_resources[$id] = 1;
        foreach ($resource['settings'] as $method_settings) {
          $enabled_resources_settings[$id]['methods'][$method_settings['method_name']] = $method_settings['method'];
          $enabled_resources_settings[$id]['supported_auth'][$method_settings['method_name']] = $method_settings['supported_auth'];
          $enabled_resources_settings[$id]['supported_formats'][$method_settings['method_name']] = $method_settings['supported_formats'];
        }
      }
    }

    $plugin_manager = drupal_container()->get('plugin.manager.rest');

    // Enable all methods and all formats for each selected resource.
    $resources = array();
    foreach ($enabled_resources as $id => $resource) {
      $methods = array_keys(array_filter($enabled_resources_settings[$id]['methods']));
      // If no HTTP methods were selected, enable all of them.
      if (!count($methods)) {
        $methods = array_keys($enabled_resources_settings[$id]['methods']);
      }
      $resources[$id] = array_fill_keys($methods, array());
      foreach ($methods as $method) {
        $auth = array_keys(array_filter($enabled_resources_settings[$id]['supported_auth'][$method]));
        if (count($auth)) {
          $resources[$id][$method]['supported_auth'] = $auth;
        }
        $formats = array_keys(array_filter($enabled_resources_settings[$id]['supported_formats'][$method]));
        if (count($formats)) {
          $resources[$id][$method]['supported_formats'] = $formats;
        }
      }
    }

    $config = config('rest.settings');
    $config->set('resources', $resources);
    $config->save();

    // Rebuild routing cache.
    drupal_container()->get('router.builder')->rebuild();
    parent::submitForm($form, $form_state);
  }

}
